<?php
/**
 * @file
 * Some functions dealing with dates.
 */

/* Format we'll use when displaying reminder times. */
define('REMINDER_D_FMT', 'Y-m-d H:i:s');

/**
 * Compute time of next reminder.
 *
 * @param reminder  $r
 *   One reminder.
 * @param string  $datefield
 *   Name of date field if known.
 *
 * @return boolean
 *   TRUE if we changed the time of next reminder.
 */
function _datereminder_get_next_reminder(&$r, $datefield = NULL) {

  // Fudge factor to deal with reminders that happen between
  // cron runs. Skew is half of the cron period in seconds.
  $skew = variable_get('datereminder_cron_frequency', 60) * 30 + 1;

  if (!isset($datefield)) {
    // Caller didn't supply datefield, so we need to find it.
    $datefield = _datereminder_get_datefield($r);
  }

  $utz = new DateTimeZone('UTC');
  $lead = $r->leadtime;

  $etime = _datereminder_get_next_occurance($datefield, $lead + $skew);

  $rem = NULL;
  if (isset($etime)) {
    $rtime = new DateTime($etime, $utz);
    $rtime->modify("-$lead seconds");
    $rem = $rtime->format(REMINDER_D_FMT);
    $r->expired = FALSE;
  }
  elseif (!$r->expired) {
    // If not already expired and there's no new date, set expire date.
    $r->expired = TRUE;
    $rem = _datereminder_now(_datereminder_retain_expired());
  }

  $ret = ($rem != $r->next);
  $r->next = $rem;
  return $ret;
}

/**
 * Get "now" in UTC as a string, with option offset from "now".
 * 
 * @param int $offset
 *   Optional offset from "now" in seconds
 * 
 * @return string
 *   Time $offset seconds in tne future as a string.
 */
function _datereminder_now($offset = 0) {
  $utz = new DateTimeZone('UTC');
  $now = new DateTime('now', $utz);
  if ($offset != 0) {
    $now->modify("+$offset seconds");
  }
  return $now->format(REMINDER_D_FMT);
}

/**
 * Return string datetime for next occurance after a suitable delay from now.
 *
 * @param field $datefield
 *   The (possibly recurring) date field from the node
 * @param int $lead
 *   How long in the future to skip. Ignore any that happen before this
 *
 * @return string
 *   String time of the next occurance. NULL means nothing past specified time.
 */
function _datereminder_get_next_occurance($datefield, $lead = 0) {
  $ptstr = _datereminder_now($lead);
  $ret = _datereminder_get_occurance_after_datestr($datefield, $ptstr);
  return $ret;
}

/**
 * Get date of next occurance in REMINDER_D_FMT format.
 *
 * Like _datereminder_get_occurance_after().
 */
function _datereminder_get_occurance_after_datestr($datefield, $datestr) {
  foreach ($datefield as $dt) {
    $etime = $dt['value'];
    if (!isset($etime)) {
      continue;
    }

    if (strcmp($etime, $datestr) > 0) {
      $ret = $etime;
      break;
    }
  }

  return $ret;
}

/**
 * Utility function - Get datefield out of node for this reminder.
 */
function _datereminder_get_datefield(&$r) {
  _datereminder_complete_reminder($r);
  return _datereminder_get_node_datefield($r->node);
}

/**
 * Utility function - Get the datefield for a reminder node.
 */
function _datereminder_get_node_datefield($node) {
  $type = $node->type;
  $dfname = variable_get("datereminder_datefield_$type", '');
  return $node->{$dfname};
}

/**
 * Return how long retired reminder should be retained.
 * 
 * @return int Time to retain expired reminder
 */
function _datereminder_retain_expired() {
  $retain = variable_get('datereminder_retain_expired', 14);
  // Convert days to seconds
  return $retain * 60 * 60 * 24;
}